﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Linq;

namespace XMLExplorer4
{
    public partial class MainForm : Form
    {
        private XmlDocument doc;
        private int currentItem = 0;
        private string path;

        public MainForm()
        {
            InitializeComponent();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            doc = new XmlDocument();
            doc.NodeChanged += new XmlNodeChangedEventHandler(doc_NodeChanged);
            doc.NodeRemoved += new XmlNodeChangedEventHandler(doc_NodeRemoved);
            doc.NodeInserted += new XmlNodeChangedEventHandler(doc_NodeInserted);
        }

        void doc_NodeInserted(object sender, XmlNodeChangedEventArgs e)
        {
            //MessageBox.Show("Node " + e.Node.Name + " added successfully!");
        }

        void doc_NodeRemoved(object sender, XmlNodeChangedEventArgs e)
        {
            //MessageBox.Show("Node " + e.Node.Name + " removed successfully!");
        }

        void doc_NodeChanged(object sender, XmlNodeChangedEventArgs e)
        {
            //string message = "Node " + e.Node.Name + " changed successfully!\n";
            //message += "Old Value: " + e.OldValue + "\n";
            //message += "New Value: " + e.NewValue + "\n";
            //MessageBox.Show(message);
        }

        private void buttonLoadBooks_Click(object sender, EventArgs e)
        {
            OpenFileDialog fd = new OpenFileDialog();
            fd.InitialDirectory = Application.StartupPath;
            DialogResult result = fd.ShowDialog();
            if (result == DialogResult.OK)
            {
                path = fd.FileName;
                LoadDocument();
            }
        }

        private void LoadDocument()
        {
            doc.Load(path);
            comboBoxBooks.Items.Clear();
            foreach (XmlNode node in doc.DocumentElement.ChildNodes)
            {
                comboBoxBooks.Items.Add(node.Attributes["bookId"].Value);
            }
        }

        private void buttonNewBook_Click(object sender, EventArgs e)
        {
            comboBoxBooks.Enabled = false;
            textBoxISBN.Text = "";
            textBoxTitle.Text = "";
            textBoxAuthor.Text = "";
            textBoxComments.Text = "";
            labelIndex.Text = "";
        }

        private void textBoxISBN_Validating(object sender, CancelEventArgs e)
        {
            if (textBoxISBN.Text != "")
            {
                errorProvider.SetError(textBoxISBN, "");
            }
            else
            {
                errorProvider.SetError(textBoxISBN, "ISBN cannot be empty!");
                e.Cancel = true;
            }
        }

        private void textBoxTitle_Validating(object sender, CancelEventArgs e)
        {
            if (textBoxTitle.Text != "")
            {
                errorProvider.SetError(textBoxTitle, "");
            }
            else
            {
                errorProvider.SetError(textBoxTitle, "Title cannot be empty!");
                e.Cancel = true;
            }
        }

        private void textBoxAuthor_Validating(object sender, CancelEventArgs e)
        {
            if (textBoxAuthor.Text != "")
            {
                errorProvider.SetError(textBoxAuthor, "");
            }
            else
            {
                errorProvider.SetError(textBoxAuthor, "Author cannot be empty!");
                e.Cancel = true;
            }
        }

        private void comboBoxBooks_SelectedIndexChanged(object sender, EventArgs e)
        {
            currentItem = int.Parse(comboBoxBooks.SelectedItem.ToString());
            PopulateControls();
        }

        private void PopulateControls()
        {
            XmlNode node = doc.DocumentElement.ChildNodes[currentItem - 1];
            comboBoxBooks.Text = node.Attributes["bookId"].Value;
            textBoxISBN.Text = node.ChildNodes[0].InnerText;
            textBoxTitle.Text = node.ChildNodes[1].InnerText;
            textBoxAuthor.Text = node.ChildNodes[2].InnerText;
            textBoxComments.Text = node.ChildNodes[3].InnerText;
            UpdateLabel();
        }

        private void UpdateLabel()
        {
            labelIndex.Text = (currentItem).ToString() + " of " +
                doc.DocumentElement.ChildNodes.Count;
        }

        private void buttonFirst_Click(object sender, EventArgs e)
        {
            currentItem = 1;
            PopulateControls();
        }

        private void buttonPrevious_Click(object sender, EventArgs e)
        {
            currentItem--;
            if (currentItem < 1)
                currentItem = 1;
            PopulateControls();
        }

        private void buttonNext_Click(object sender, EventArgs e)
        {
            currentItem++;
            if (currentItem >= doc.DocumentElement.ChildNodes.Count)
                currentItem = doc.DocumentElement.ChildNodes.Count;
            PopulateControls();
        }

        private void buttonLast_Click(object sender, EventArgs e)
        {
            currentItem = doc.DocumentElement.ChildNodes.Count;
            PopulateControls();
        }

        private void buttonDelete_Click(object sender, EventArgs e)
        {
            XmlNode node = doc.SelectSingleNode("//book[@bookId='" +
                comboBoxBooks.SelectedItem + "']");
            if (node != null)
                doc.DocumentElement.RemoveChild(node);
            doc.Save(Application.StartupPath + "/books.xml");
            LoadDocument();
            currentItem--;
            if (currentItem < 1)
                currentItem = 1;
            PopulateControls();
            MessageBox.Show("Record has been deleted!");
        }

        private void buttonUpdate_Click(object sender, EventArgs e)
        {
            XmlNode node = doc.DocumentElement.SelectSingleNode(
                "//book[@bookId='" + comboBoxBooks.SelectedItem + "']");
            if (node != null)
            {
                node.ChildNodes[0].InnerText = textBoxISBN.Text;
                node.ChildNodes[1].InnerText = textBoxTitle.Text;
                node.ChildNodes[2].InnerText = textBoxAuthor.Text;
                XmlCDataSection comments = doc.CreateCDataSection(
                    textBoxComments.Text);
                node.ChildNodes[3].ReplaceChild(comments,
                    node.ChildNodes[3].ChildNodes[0]);
                doc.Save(Application.StartupPath + "/books.xml");
                MessageBox.Show("Record has been updated!");
                LoadDocument();
            }
        }

        private void buttonAdd_Click(object sender, EventArgs e)
        {
            XmlElement book = doc.CreateElement("book");
            XmlElement isbn = doc.CreateElement("isbn");
            XmlElement title = doc.CreateElement("title");
            XmlElement author = doc.CreateElement("author");
            XmlElement comments = doc.CreateElement("comments");
            XmlAttribute id = doc.CreateAttribute("bookId");
            id.Value = GetValue();

            XmlText isbnText = doc.CreateTextNode(textBoxISBN.Text);
            XmlText titleText = doc.CreateTextNode(textBoxTitle.Text);
            XmlText authorText = doc.CreateTextNode(textBoxAuthor.Text);
            XmlCDataSection commentsText = 
                doc.CreateCDataSection(textBoxComments.Text);

            isbn.AppendChild(isbnText);
            title.AppendChild(titleText);
            author.AppendChild(authorText);
            comments.AppendChild(commentsText);

            book.Attributes.Append(id);
            book.AppendChild(isbn);
            book.AppendChild(title);
            book.AppendChild(author);
            book.AppendChild(comments);

            doc.DocumentElement.AppendChild(book);
            doc.Save(Application.StartupPath + "/books.xml");

            comboBoxBooks.Enabled = true;
            currentItem = int.Parse(id.Value);
            if (!string.IsNullOrEmpty(path))
                LoadDocument();
            UpdateLabel();
        }

        private string GetValue()
        {
            XDocument bookDoc = XDocument.Load(Application.StartupPath +
                "/books.xml");
            var ids = from book in bookDoc.Descendants().Elements("book")
                      select int.Parse(book.Attribute("bookId").Value);
            int newId = ids.Max();
            return (++newId).ToString();
        }
    }
}
